/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.portal.admin.controller;

import se.vgregion.portal.cs.domain.SiteKey;

import java.io.Serializable;
import java.util.*;

public class SiteKeyHelper {
    private final List<SiteKey> siteKeys;

    /**
     * Constructor.
     *
     * @param value List of {@link SiteKey}s
     */
    public SiteKeyHelper(List<SiteKey> value) {
        this.siteKeys = value;
    }

    /**
     * Constructor.
     *
     * @param value Collection of {@link SiteKey}s
     */
    public SiteKeyHelper(Collection<SiteKey> value) {
        this(new ArrayList<SiteKey>(value));
    }

    /**
     * Get the {@link SiteKey}s.
     *
     * @return the {@link SiteKey}s
     */
    public List<SiteKey> get() {
        return siteKeys;
    }

    /**
     * Creates a new instance of {@link SiteKeyHelper} where the site key order is determined by the site key attribute.
     *
     * @return a new instance of {@link SiteKeyHelper}
     */
    public SiteKeyHelper orderBySiteKey() {
        Collections.sort(siteKeys, new SiteKeyComparator());
        return new SiteKeyHelper(siteKeys);
    }

    private static class SiteKeyComparator implements Comparator<SiteKey>, Serializable {
        private static final long serialVersionUID = -4400606224805136030L;

        @Override
        public int compare(SiteKey one, SiteKey other) {
            return one.getSiteKey().toLowerCase().compareTo(other.getSiteKey().toLowerCase());
        }
    }

    /**
     * Creates a new instance of {@link SiteKeyHelper} with only active {@link SiteKey}s.
     *
     * @return a new instance of {@link SiteKeyHelper}
     */
    public SiteKeyHelper filterActive() {
        for (Iterator<SiteKey> it = siteKeys.iterator(); it.hasNext();) {
            SiteKey siteKey = it.next();
            if (!siteKey.getActive()) {
                it.remove();
            }
        }
        return new SiteKeyHelper(siteKeys);
    }

    /**
     * Creates a new instance of {@link SiteKeyHelper} where the {@link SiteKey}s' descriptions are cut so that they do
     * not exceed the length of <code>maxLen</code>.
     *
     * @param maxLen the maximum length of the {@link SiteKey}s' descriptions.
     * @return a new instance of {@link SiteKeyHelper}
     */
    public SiteKeyHelper descriptionElipsis(int maxLen) {
        for (SiteKey siteKey : siteKeys) {
            siteKey.setDescription(ellipsis(siteKey.getDescription(), maxLen));
        }
        return new SiteKeyHelper(siteKeys);
    }

    private String ellipsis(String text, int len) {
        final int three = 3;
        if (text.length() > Math.max(three, len)) {
            return text.substring(0, len - three) + "...";
        } else {
            return text;
        }
    }
}
